﻿#include <stdlib.h>
#include <string.h>
#include <iostream>
#include <chrono>
#include <stdexcept>
#include <thread>
#include <direct.h>
#include <string>

#include "pvr_Platform.h"

void initEnv();
void loop();
void exit();
void PVRMessages();
void processInitError(pvrPlatformResult ret);
void processGetLoggedInUser(pvrMessageHandle message);
void processGetAccessToken(pvrMessageHandle message);
void processCheckEntitlement(pvrMessageHandle message);
void processGetCurrentLanguage(pvrMessageHandle message);
void processNotifyLogout(pvrMessageHandle message);
void processNotifyRuntimeError(pvrMessageHandle message);

static std::atomic_bool app_run = false;

int main(int argc, char* argv[]) {
  pvrID appid;
  printf("Please enter appid：\n");
  //    Just for the convenience of testing, please 
  //use the real appid initialization in the actual project 
  scanf_s("%llu", &appid);
  printf("starting service...\n");
  pvrPlatformResult initret = pvr_PlatformInit(appid);
  if (initret != pvrPlatformResult::pvrPlatformResult_Success) {
    processInitError(initret);
    printf("init Fail\n");
    return 1;
  }

  app_run = true;
  initEnv();
  loop();
  pvr_PlatformShutdown();
  return 0;
}

void initEnv() {
  pvr_CheckEntitlement();
  pvr_GetCurrentLanguage();
  pvr_GetAccessToken();
  pvr_GetLoggedInUser();
}

void loop() {
  while (app_run) {
    //do something
    PVRMessages();
    std::this_thread::sleep_for(std::chrono::seconds(2));
  }
}

void exit() {
  app_run = false;
}

void PVRMessages() {
  pvrMessageHandle message = nullptr;
  while ((message = pvr_PollMessage()) != nullptr) {
    switch (pvr_Message_GetType(message)) {
    case pvrMessageType::pvrMessage_CheckEntitlement:
      processCheckEntitlement(message);
      break;
    case pvrMessageType::pvrMessage_GetAccessToken:
      processGetAccessToken(message);
      break;
    case pvrMessageType::pvrMessage_GetLoggedInUser:
      processGetLoggedInUser(message);
      break;
    case pvrMessageType::pvrMessage_GetCurrentLanguage:
      processGetCurrentLanguage(message);
      break;
    case pvrMessageType::pvrMessage_Notify_Logout:
      processNotifyLogout(message);
      break;
    case pvrMessageType::pvrMessage_Notify_RuntimeError:
      processNotifyRuntimeError(message);
      break;
    default:
      printf("Issues not addressed: %d\n", pvr_Message_GetType(message));
      break;
    }
  }
}

void processInitError(pvrPlatformResult ret) {
  switch (ret) {
    case pvrPlatformResult_Success:
      printf("Success\n");
      break;
    case pvrPlatformResult_Failed:
      printf("Failed\n");
      break;
    case pvrPlatformResult_NotReady:
      printf("Not Ready\n");
      break;
    case pvrPlatformResult_ServiceNotInstalled:
      printf("Service Not Installed\n");
      break;
    case pvrPlatformResult_ServiceLaunchFailed:
      printf("Service Launch Failed\n");
      break;
    case pvrPlatformResult_ConnectServiceFailed:
      printf("Connect Service Failed\n");
      break;
    case pvrPlatformResult_Unknown:
      printf("Unknown\n");
      break;
    default:
      printf("Other error\n");
      break;
  }
}

void processCheckEntitlement(pvrMessageHandle message) {
  if (!pvr_Message_IsError(message)) {
    printf("User has an entitlement: %d\n", pvr_CheckEntitlement_GetResult(message));
  }
  else {
    const pvrErrorHandle error = pvr_Message_GetError(message);
    printf("Could NOT get an entitlement  failure: %s\n", pvr_Message_GetErrorInfo(error));
  }
}

void processGetAccessToken(pvrMessageHandle message) {
  if (!pvr_Message_IsError(message)) {
    printf("Get AccessToken successfully  AccessToken: %s\n", pvr_Message_GetString(message));
  }
  else {
    const pvrErrorHandle error = pvr_Message_GetError(message);
    printf("Get AccessToken  failure: %s\n", pvr_Message_GetErrorInfo(error));
  }
}

void processGetLoggedInUser(pvrMessageHandle message) {
  if (!pvr_Message_IsError(message)) {
    printf("Get Logged In User successfully userid:%lld,username:%s\n",
      pvr_UserInfo_GetUserId(message), pvr_UserInfo_GetUserName(message));
  }
  else {
    const pvrErrorHandle error = pvr_Message_GetError(message);
    printf("Get Logged In User failure: %s\n", pvr_Message_GetErrorInfo(error));
  }
}

void processGetCurrentLanguage(pvrMessageHandle message) {
  if (!pvr_Message_IsError(message)) {
    printf("Get Current Language In User successfully  CurrentLanguage: %s\n", pvr_Message_GetString(message));
  }
  else {
    const pvrErrorHandle error = pvr_Message_GetError(message);
    printf("Get Current Language In User failure: %s\n", pvr_Message_GetErrorInfo(error));
  }
}

void processNotifyLogout(pvrMessageHandle message) {
  if (!pvr_Message_IsError(message)) {
    printf("The login status is invalid, exit the app....\n");
  }
}

void processNotifyRuntimeError(pvrMessageHandle message) {
  if (!pvr_Message_IsError(message)) {
    switch (pvr_RuntimeError_GetError(message)) {
      case pvrRuntimeError_Refund:
        printf("User the game has been refunded, exit the app....\n");
        break;
      case pvrRuntimeError_TooMuchDevices:
        printf(
            "The user is logged into the app on another device, exit the "
            "app....\n");
        break;
      case pvrRuntimeError_Unknown:
        printf("Some unknown error has occurred, exit the app....\n");
        break;
      default:
        printf(
            "Please update the sdk to receive the latest messages, exit the "
            "app....\n");
        break;
    }
    exit();
  }
}
